@@ -29,6 +29,13 @@ class ListInstance(Lister):
             action='store_true'
         )
 
+        parser.add_argument(
+            '--id',
+            nargs='+',
+            help='list of instance IDs to filter, e.g --id 1 2 3 ',
+            required=False
+        )
+
         return parser
 
     def get_data(self, parsed_args):
@@ -43,7 +50,7 @@ class ListInstance(Lister):
         elif parsed_args.advanced:
             query_params['type'] = 'advanced'
 
-        data = self.app.session.get( '/instances', params=query_params)
+        data = self.app.session.get('/instances', params=query_params)
         return data
 
 
@@ -69,12 +76,38 @@ class CreateInstance(ShowOne):
             metavar='<instance_type>',
             help='InstanceType name or ID'
         )
+        parser.add_argument(
+            '--root-volume-size',
+            required=False,
+            metavar='<root_volume_size>',
+            help=('Optionally you can set the size of root volume, otherwise root volume '
+                  'will be created according to the instance type')
+        )
+
+        parser.add_argument(
+            '--root-volume-type',
+            required=False,
+            metavar='<root_volume_type>',
+            choices=['SSD', 'ULTRA_DISK'],
+            type=lambda value: str(value).upper().replace("-", "_"),
+            help=(
+                'Optionally you can set the type of root volume, '
+                'could be SSD or ULTRA_DISK case insensitive, the default is SSD'
+            )
+        )
+
         parser.add_argument(
             '--image',
-            required=True,
+            required=False,
             metavar='<image>',
             help='Image name or ID'
         )
+        parser.add_argument(
+            '--snapshot',
+            required=False,
+            metavar='<snapshot>',
+            help='Snapshot name or ID'
+        )
         parser.add_argument(
             '--network',
             required=False,
@@ -129,6 +162,14 @@ class CreateInstance(ShowOne):
             action='store_true'
         )
 
+        parser.add_argument(
+            '--count',
+            metavar='<count>',
+            type=int,
+            help='Create multiple instance at once',
+            required=False
+        )
+
         return parser
 
     def _check_arg_validity(self, parsed_args):
@@ -153,11 +194,32 @@ class CreateInstance(ShowOne):
                 not parsed_args.simple and parsed_args.network is None,
                 "--network is mandatory in advanced mode",
             ),
-
             (
                 parsed_args.with_public_ip and parsed_args.public_ip is not None,
                 "--with-public-ip and --public-ip must not be used together",
             ),
+            # Creating multiple instance conditions:
+            (
+                parsed_args.with_public_ip and parsed_args.count is not None,
+                "--with-public-ip and --count must not be used together",
+            ),
+            (
+                parsed_args.public_ip and parsed_args.count is not None,
+                "--public-ip and --count must not be used together",
+            ),
+            (
+                parsed_args.additional_volumes and parsed_args.count is not None,
+                "using --additional-volumes and --count together is not supported",
+            ),
+
+            (
+                parsed_args.image and parsed_args.snapshot,
+                "--image and --snapshot must not be used together",
+            ),
+            (
+                parsed_args.image is None and parsed_args.snapshot is None,
+                "one of the --image and --snapshot arguments must be used",
+            ),
 
         ]
         errors = []
@@ -169,12 +231,18 @@ class CreateInstance(ShowOne):
 
     def get_data(self, parsed_args):
         self._check_arg_validity(parsed_args)
+        if parsed_args.count and parsed_args.count > 1:
+            return self.create_instance_group(parsed_args)
+
         payload = {
             'name': parsed_args.name,
             'instance_type': parsed_args.instance_type,
-            'image': parsed_args.image,
             'type': 'simple' if parsed_args.simple else 'advanced',
         }
+        if parsed_args.image:
+            payload['image'] = parsed_args.image
+        if parsed_args.snapshot:
+            payload['snapshot'] = parsed_args.snapshot
         if parsed_args.network:
             payload['network'] = parsed_args.network
         if parsed_args.ssh_key:
@@ -195,9 +263,49 @@ class CreateInstance(ShowOne):
         if parsed_args.description:
             payload['description'] = parsed_args.description
 
+        if parsed_args.root_volume_type:
+            payload['root_volume_type'] = parsed_args.root_volume_type
+
+        if parsed_args.root_volume_size:
+            payload['root_volume_size'] = parsed_args.root_volume_size
+
         data = self.app.session.post('/instances', payload)
         return data
 
+    def create_instance_group(self, parsed_args):
+        self.schema = schemas.InstanceGroupDetails(many=False)
+        payload = {
+            'name': parsed_args.name,
+            "count": parsed_args.count,
+            "template": {
+                'instance_type': parsed_args.instance_type,
+                'type': 'simple' if parsed_args.simple else 'advanced',
+            },
+        }
+        if parsed_args.image:
+            payload['template']['image'] = parsed_args.image
+        if parsed_args.snapshot:
+            payload['template']['snapshot'] = parsed_args.snapshot
+        if parsed_args.network:
+            payload['template']['network'] = parsed_args.network
+        if parsed_args.ssh_key:
+            payload['template']['ssh_key_name'] = parsed_args.ssh_key
+        if parsed_args.firewall:
+            payload['template']['security_groups'] = parsed_args.firewall
+        if parsed_args.description:
+            payload['description'] = parsed_args.description
+        if parsed_args.root_volume_type:
+            payload['template']['root_volume_type'] = parsed_args.root_volume_type
+        if parsed_args.root_volume_size:
+            payload['template']['root_volume_size'] = parsed_args.root_volume_size
+
+        data = self.app.session.post('/instance-groups', payload)
+        command = "[bold blue]digicloud instance list --id %s[/bold blue]" % " ".join(
+            data['instance_ids']
+        )
+        self.app.console.print(f"Check you new instances via `%s`" % command)
+        return data
+
 
 class ShowInstance(ShowOne):
     """Show instance details."""
@@ -543,7 +651,7 @@ class UpdateInstance(ShowOne):
         return data
 
     def _on_400(self, parsed_args, response):
-        error_msg = response.json()['errors']
+        error_msg = response.json()['message']
         if 'stop your instance' in error_msg:
             return CLIError([
                 dict(
@@ -555,6 +663,15 @@ class UpdateInstance(ShowOne):
                 )
             ])
 
+    def _on_404(self, parsed_args, response):
+        return CLIError([
+            dict(
+                msg="Please check your instance name and try again!",
+                hint="You can list your instances by running: "
+                     "[bold blue]digicloud instance list [/bold blue]"
+            )
+        ])
+
 
 class ResizeInstance(ShowOne):
     """Resize your instance"""
@@ -588,7 +705,7 @@ class ResizeInstance(ShowOne):
         return self.app.session.patch(uri, payload)
 
     def _on_400(self, parsed_args, response):
-        error_msg = response.json()['errors']
+        error_msg = response.json()['message']
         if 'stop your instance' in error_msg:
             return CLIError([
                 dict(
@@ -666,3 +783,22 @@ class RemoveFirewall(Command):
         uri = '/instances/%s/security-groups/%s' % (parsed_args.instance,
                                                     parsed_args.firewall)
         self.app.session.delete(uri)
+
+
+class ListInstanceSnapshot(Lister):
+    """List Instance Snapshots"""
+    schema = schemas.SnapshotList(many=True)
+
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+        parser.add_argument(
+            'instance',
+            metavar='<instance>',
+            help='Instance name or ID'
+        )
+        return parser
+
+    def get_data(self, parsed_args):
+        uri = '/instances/%s/snapshots' % parsed_args.instance
+        data = self.app.session.get(uri)
+        return data
